DECLARE SUB LoadObject (File$)
DECLARE SUB DrawVector ()
DECLARE SUB CreateLookupTables ()
DECLARE SUB CreateVectorObject ()
'Vector Balls by Bill McDonald
'           Thanx to Tumblin / Bodies In Motion
'           For some 3d info

DEFINT A-Z
'$INCLUDE: 'directqb.bi'

' maximum number of balls
CONST maxballs = 8 '8 for box, 7 for other shape
CONST Shape = 1 '1 for Box, 2 for Shape
CONST Pi = 3.14159265#
CONST OrigDist& = 256


' sin and cos lookup tables
DIM SHARED cosine(360) AS DOUBLE
DIM SHARED sine(360) AS DOUBLE
DIM SHARED xangle, yangle, zangle, distance, Direction
DIM SHARED Balls(DQBsize(0, 0, 25, 25), maxballs)
DIM SHARED Pal AS STRING * 768

' structure for one vector ball
TYPE VectBallType
   x AS INTEGER
   y AS INTEGER
   z AS INTEGER
   colour AS INTEGER
END TYPE

'original vector ball object
DIM SHARED object(maxballs) AS VectBallType
DIM SHARED temp(maxballs) AS VectBallType
DIM SHARED dummy AS VectBallType

' start off with some random angles
xangle = INT(RND * 360)
yangle = INT(RND * 360)
zangle = INT(RND * 360)

'the z distance from the center of the universe (0, 0, 0) :-)
distance = INT(OrigDist&)

'tell when the object is moving towards or away from you
Direction = 1

RANDOMIZE TIMER
SCREEN 13

CreateLookupTables
CreateVectorObject

A = DQBinit(3, 0)
IF A <> 0 THEN
  DQBclose
  PRINT "Init Problem: ", A
  END
END IF

A = DQBloadLayer(2, "balls.bmp", Pal)
IF A <> 0 THEN
  DQBclose
  PRINT "Error Loading BALLS.BMP : ", A
  END
END IF
DQBsetPal Pal 'uses colors 1-64

FOR I = 1 TO maxballs
 DQBget 2, 0, 0, 25, 25, VARSEG(Balls(0, I)), VARPTR(Balls(0, I))
NEXT I
DQBclearLayer 1

FOR I = 64 TO 96
 DQBsetCol I, I - 32, 0, 0
NEXT I
FOR I = 96 TO 128
 DQBsetCol I, 63 - (I - 32), 0, 0
NEXT I
FOR I = 32 TO 180
  DQBellipse 3, 160, 100, I, I - 2, (I MOD 64) + 64
  DQBellipse 3, 161, 100, I, I - 2, (I MOD 64) + 64
NEXT I

COLOR 65
ST! = TIMER
DO
  IF INKEY$ <> "" THEN quit = 1
 
  IF Direction = 1 THEN      ' if object is moving away from you,
    distance = distance + 1   ' move it even further
    IF (distance >= 600) THEN  ' if at the limit, make it come back
      Direction = 0
      distance = 600
    END IF
  ELSE   ' no, object is moving towards you
    distance = distance - 1     ' bring it a little closer
    IF distance <= 256 THEN     ' if at limit, make it go away again
      Direction = 1
      distance = 256
    END IF
  END IF
        
 'update the angles
  xangle = xangle + 5

  IF xangle >= 360 THEN
    xangle = 0
    yangle = yangle + 5
  END IF
  IF yangle >= 360 THEN
    yangle = 0
    zangle = zangle - 5
  END IF
  IF zangle <= 0 THEN zangle = 359

  'draw the vector ball object on the hidden screen
  DrawVector
 
  DQBgetCol 64, Ored, Og, Ob
  FOR I = 64 TO 127
    DQBgetCol I + 1, R, G, B
    DQBsetCol I, R, G, B
  NEXT I
  DQBsetCol 128, Ored, Og, Ob
LOOP UNTIL quit = 1
       
SCREEN 0: WIDTH 80: COLOR 7
PRINT "by Bill McDonald"
A$ = INPUT$(1)

DQBclose
END

Box:
'back top left
DATA -30,-30,30
' back top right
DATA 30,-30, 31
' back bottom left
DATA -30, 30, 32
' back bottom right
DATA 30, 30, 33
' front top left
DATA -30,-30, -30
' front top right
DATA 30, -30, -31
' front bottom left
DATA -30, 30, -32
' front bottom right
DATA 30, 30, -33


Lines:

'DATA -100,0,0
'DATA -50,0,0
DATA 0,0,0
'DATA 50,0,0
'DATA 100,0,0


DATA 0,70,0
DATA 0,-70,0

DATA 50,70,50
DATA -50,-70,-50
'
DATA 80,70,80
DATA -80,-70,-80

REM $DYNAMIC
'---------------------------------------------------------------
SUB CreateLookupTables
FOR I = 0 TO 360
  cosine(I) = COS(I * Pi / 180) 'coverts radians to degrees and makes table
  sine(I) = SIN(I * Pi / 180)
NEXT I
END SUB

'---------------------------------------------------------------
SUB CreateVectorObject
' This object will look like a cube
IF Shape = 1 THEN
  RESTORE Box:
  FOR I = 0 TO maxballs - 1
    READ object(I).x, object(I).y, object(I).z
  NEXT I
END IF

IF Shape = 2 THEN
  RESTORE Lines:
  FOR I = 0 TO maxballs - 1
    READ object(I).x, object(I).y, object(I).z
  NEXT I
END IF

IF T = 5 THEN
' back top left
  object(0).x = -30
  object(0).y = -30
  object(0).z = 30

' back top right
  object(1).x = 30
  object(1).y = -30
  object(1).z = 31
' back bottom left
  object(2).x = -30
  object(2).y = 30
  object(2).z = 32
' back bottom right
  object(3).x = 30
  object(3).y = 30
  object(3).z = 33
' front top left
  object(4).x = -30
  object(4).y = -30
  object(4).z = -30
' front top right
  object(5).x = 30
  object(5).y = -30
  object(5).z = -31
' front bottom left
  object(6).x = -30
  object(6).y = 30
  object(6).z = -32
' front bottom right
  object(7).x = 30
  object(7).y = 30
  object(7).z = -33
END IF

'Color
FOR I = 0 TO maxballs - 1
 object(I).colour = I + 1
NEXT I
END SUB

'---------------------------------------------------------------
SUB DrawVector
' first we have to do all the rotations

FOR I = 0 TO maxballs - 1
' rotate around the x-axis
  ny = INT(object(I).y * cosine(xangle) - object(I).z * sine(xangle))
  nz = INT(object(I).y * sine(xangle) + object(I).z * cosine(xangle))
  nx = INT(object(I).x)
  temp(I).x = nx
  temp(I).y = ny
  temp(I).z = nz

' rotate around the y-axis
  nx = INT(temp(I).x * cosine(yangle) + temp(I).z * sine(yangle))
  nz = INT(-temp(I).x * sine(yangle) + temp(I).z * cosine(yangle))
  temp(I).x = nx
  temp(I).z = nz

' rotate around the z-axis
  nx = INT(temp(I).x * cosine(zangle) - temp(I).y * sine(zangle))
  ny = INT(temp(I).x * sine(zangle) + temp(I).y * cosine(zangle))
  temp(I).x = nx
  temp(I).y = ny

' now we have to push the z coordinates into the view area
  temp(I).z = temp(I).z - distance

' finally we project & copy the new x,y,z to a temporary array
' NOTE- Same as with 3d starfield

  temp(I).x = ((OrigDist& * temp(I).x) \ temp(I).z) + 160
  temp(I).y = ((OrigDist& * temp(I).y) \ temp(I).z) + 100
  temp(I).colour = object(I).colour
NEXT I

' now we need to do z-buffering (higher z farther the distance,
' put that object first)
' the ball z with most positive value should be in lowest index
' I used bubble sort here because there are only 8 balls

FOR j = 0 TO maxballs - 2
  FOR k = j + 1 TO maxballs - 1
    IF temp(j).z >= temp(k).z THEN  'swap the two vector balls x,y,z, Colour
     SWAP temp(j).z, temp(k).z
     SWAP temp(j).x, temp(k).x
     SWAP temp(j).y, temp(k).y
     SWAP temp(j).colour, temp(k).colour
    END IF
  NEXT k
NEXT j

' we're done everything, so draw the object!
' first we erase the old stuff off the drawing zone
DQBclearLayer 2
       

FOR I = 0 TO maxballs - 1
 DQBput 2, temp(I).x - 13, temp(I).y - 13, VARSEG(Balls(0, temp(I).colour)), VARPTR(Balls(0, temp(I).colour))
NEXT I
   
DQBcopyLayer 3, 1
DQBcopyTransLayer 2, 1
DQBwait 1
DQBcopyLayer 1, 0
END SUB

